@category = N_("Toolkit")
@title = N_("SlideBackgroundImage")
@abstract = N_("Toolkit to display an image as a background of slides")
@description = N_("Displays an image as a background of slides.")
@parameters = {
  "@slide_background_image" => {
    :default => N_("(Must be specified.)"),
    :description => N_("Image file name."),
  },
}
