# Copyright (C) 2004-2014  Kouhei Sutou <kou@cozmixng.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

require "rabbit/utils"

require "rabbit/logger/stderr"
require "rabbit/logger/gui"

module Rabbit
  module Logger
    extend Utils

    class << self
      def types
        collect_classes_under_module(self)
      end

      def default
        if Utils.support_console_output?
          Logger::STDERR.new
        else
          Logger::GUI.new
        end
      end
    end
  end
end
